@@ -1,3 +1,4 @@
+
 """
 ********************************************************************************
 compas_blender.ui
@@ -5,6 +6,8 @@ compas_blender.ui
 
 .. currentmodule:: compas_blender.ui
 
+.. autosummary::
+    :toctree: generated/
 
 """
 from __future__ import absolute_import
@@ -15,4 +18,10 @@ from .command import *
 from .controller import *
 from .mouse import *
 
-__all__ = []
+from .bui import __all__ as a
+from .button import __all__ as b
+from .command import __all__ as c
+from .controller import __all__ as d
+from .mouse import __all__ as e
+
+__all__ = a + b + c + d + e
